@@ -38,7 +38,7 @@
  * any kind of modification.
  */
 
-#include "clutter/clutter-build-config.h"
+#include "config.h"
 
 #include "clutter/clutter-actor-meta-private.h"
 
@@ -208,7 +208,8 @@ clutter_actor_meta_class_init (ClutterActorMetaClass *klass)
   obj_props[PROP_ACTOR] =
     g_param_spec_object ("actor", NULL, NULL,
                          CLUTTER_TYPE_ACTOR,
-                         CLUTTER_PARAM_READABLE |
+                         G_PARAM_READABLE |
+                         G_PARAM_STATIC_STRINGS |
                          G_PARAM_EXPLICIT_NOTIFY);
 
   /**
@@ -219,7 +220,8 @@ clutter_actor_meta_class_init (ClutterActorMetaClass *klass)
   obj_props[PROP_NAME] =
     g_param_spec_string ("name", NULL, NULL,
                          NULL,
-                         CLUTTER_PARAM_READWRITE);
+                         G_PARAM_READWRITE |
+                         G_PARAM_STATIC_STRINGS);
 
   /**
    * ClutterActorMeta:enabled:
@@ -229,7 +231,8 @@ clutter_actor_meta_class_init (ClutterActorMetaClass *klass)
   obj_props[PROP_ENABLED] =
     g_param_spec_boolean ("enabled", NULL, NULL,
                           TRUE,
-                          CLUTTER_PARAM_READWRITE);
+                          G_PARAM_READWRITE |
+                          G_PARAM_STATIC_STRINGS);
 
   gobject_class->finalize = clutter_actor_meta_finalize;
   gobject_class->set_property = clutter_actor_meta_set_property;
@@ -352,7 +355,7 @@ clutter_actor_meta_get_enabled (ClutterActorMeta *meta)
  * Sets or unsets a back pointer to the #ClutterActor that owns
  * the @meta
  */
-void
+static void
 _clutter_actor_meta_set_actor (ClutterActorMeta *meta,
                                ClutterActor     *actor)
 {
@@ -403,7 +406,7 @@ _clutter_actor_meta_set_priority (ClutterActorMeta *meta,
   priv->priority = priority;
 }
 
-gint
+static gint
 _clutter_actor_meta_get_priority (ClutterActorMeta *meta)
 {
   ClutterActorMetaPrivate *priv;
@@ -415,7 +418,7 @@ _clutter_actor_meta_get_priority (ClutterActorMeta *meta)
   return priv->priority;
 }
 
-gboolean
+static gboolean
 _clutter_actor_meta_is_internal (ClutterActorMeta *meta)
 {
   ClutterActorMetaPrivate *priv =
@@ -430,7 +433,7 @@ _clutter_actor_meta_is_internal (ClutterActorMeta *meta)
  * ClutterMetaGroup: a collection of ClutterActorMeta instances
  */
 
-G_DEFINE_TYPE (ClutterMetaGroup, _clutter_meta_group, G_TYPE_OBJECT);
+G_DEFINE_FINAL_TYPE (ClutterMetaGroup, _clutter_meta_group, G_TYPE_OBJECT);
 
 static void
 _clutter_meta_group_dispose (GObject *gobject)
