@@ -23,10 +23,9 @@
  */
 
 /**
- * SECTION:clutter-constraint
- * @Title: ClutterConstraint
- * @Short_Description: Abstract class for constraints on position or size
- * @See_Also: #ClutterAction
+ * ClutterConstraint:
+ * 
+ * Abstract class for constraints on position or size
  *
  * #ClutterConstraint is a base abstract class for modifiers of a #ClutterActor
  * position or size.
@@ -35,9 +34,7 @@
  * the position or size of the #ClutterActor to which it is applied, by
  * updating the actor's allocation. Each #ClutterConstraint can change the
  * allocation of the actor to which they are applied by overriding the
- * #ClutterConstraintClass.update_allocation() virtual function.
- *
- * #ClutterConstraint is available since Clutter 1.4
+ * [vfunc@Clutter.Constraint.update_allocation] virtual function.
  *
  * ## Using Constraints
  *
@@ -51,7 +48,7 @@
  * #ClutterBoxLayout lay out their children.
  *
  * Constraints are attached to a #ClutterActor, and are available
- * for inspection using clutter_actor_get_constraints().
+ * for inspection using [method@Clutter.Actor.get_constraints].
  *
  * Clutter provides different implementation of the #ClutterConstraint
  * abstract class, for instance:
@@ -67,35 +64,6 @@
  *  horizontal or vertical edges then it can also expand to fit the empty
  *  space.
  *
- * The [constraints example](https://git.gnome.org/browse/clutter/tree/examples/constraints.c?h=clutter-1.18)
- * uses various types of #ClutterConstraints to lay out three actors on a
- * resizable stage. Only the central actor has an explicit size, and no
- * actor has an explicit position.
- *
- *  - The #ClutterActor with #ClutterActor:name `layerA` is explicitly
- *  sized to 100 pixels by 25 pixels, and it's added to the #ClutterStage
- *  - two #ClutterAlignConstraints are used to anchor `layerA` to the
- *  center of the stage, by using 0.5 as the alignment #ClutterAlignConstraint:factor on
- *  both the X and Y axis
- *  - the #ClutterActor with #ClutterActor:name `layerB` is added to the
- *  #ClutterStage with no explicit size
- *  - the #ClutterActor:x and #ClutterActor:width of `layerB` are bound
- *  to the same properties of `layerA` using two #ClutterBindConstraint
- *  objects, thus keeping `layerB` aligned to `layerA`
- *  - the top edge of `layerB` is snapped together with the bottom edge
- *  of `layerA`; the bottom edge of `layerB` is also snapped together with
- *  the bottom edge of the #ClutterStage; an offset is  given to the two
- *  #ClutterSnapConstraintss to allow for some padding; since `layerB` is
- *  snapped between two different #ClutterActors, its height is stretched
- *  to match the gap
- *  - the #ClutterActor with #ClutterActor:name `layerC` mirrors `layerB`,
- *  snapping the top edge of the #ClutterStage to the top edge of `layerC`
- *  and the top edge of `layerA` to the bottom edge of `layerC`
- *
- * You can try resizing interactively the #ClutterStage and verify
- * that the three #ClutterActors maintain the same position and
- * size relative to each other, and to the #ClutterStage.
- *
  * It is important to note that Clutter does not avoid loops or
  * competing constraints; if two or more #ClutterConstraints
  * are operating on the same positional or dimensional attributes of an
@@ -105,7 +73,7 @@
  * ## Implementing a ClutterConstraint
  *
  * Creating a sub-class of #ClutterConstraint requires the
- * implementation of the #ClutterConstraintClass.update_allocation()
+ * implementation of the [vfunc@Clutter.Constraint.update_allocation]
  * virtual function.
  *
  * The `update_allocation()` virtual function is called during the
@@ -128,15 +96,15 @@
  * can be recovered at any point using clutter_actor_meta_get_actor().
  */
 
-#include "clutter-build-config.h"
+#include "config.h"
 
 #include <string.h>
 
-#include "clutter-constraint-private.h"
+#include "clutter/clutter-constraint-private.h"
 
-#include "clutter-actor.h"
-#include "clutter-actor-meta-private.h"
-#include "clutter-private.h"
+#include "clutter/clutter-actor.h"
+#include "clutter/clutter-actor-meta-private.h"
+#include "clutter/clutter-private.h"
 
 G_DEFINE_ABSTRACT_TYPE (ClutterConstraint,
                         clutter_constraint,
