@@ -472,23 +472,6 @@ typedef enum /*< prefix=CLUTTER_EFFECT_PAINT >*/
   CLUTTER_EFFECT_PAINT_BYPASS_EFFECT = (1 << 1)
 } ClutterEffectPaintFlags;
 
-
-/**
- * ClutterLongPressState:
- * @CLUTTER_LONG_PRESS_QUERY: Queries the action whether it supports
- *   long presses
- * @CLUTTER_LONG_PRESS_ACTIVATE: Activates the action on a long press
- * @CLUTTER_LONG_PRESS_CANCEL: The long press was cancelled
- *
- * The states for the #ClutterClickAction::long-press signal.
- */
-typedef enum /*< prefix=CLUTTER_LONG_PRESS >*/
-{
-  CLUTTER_LONG_PRESS_QUERY,
-  CLUTTER_LONG_PRESS_ACTIVATE,
-  CLUTTER_LONG_PRESS_CANCEL
-} ClutterLongPressState;
-
 /**
  * ClutterEventFlags:
  * @CLUTTER_EVENT_NONE: No flag set
@@ -534,6 +517,17 @@ typedef enum /*< flags prefix=CLUTTER_EVENT >*/
  *   cancelled when the finger(s) move past a certain threshold.
  * @CLUTTER_PROXIMITY_IN: A tool entered in proximity to a tablet;
  * @CLUTTER_PROXIMITY_OUT: A tool left from the proximity area of a tablet;
+ * @CLUTTER_PAD_BUTTON_PRESS: A tablet pad button press
+ * @CLUTTER_PAD_BUTTON_RELEASE: A tablet pad button release
+ * @CLUTTER_PAD_STRIP: A tablet pad tactile strip event
+ * @CLUTTER_PAD_RING: A tablet pad tactile ring event
+ * @CLUTTER_PAD_DIAL: A tablet pad dial event
+ * @CLUTTER_DEVICE_ADDED: Notification of an added device
+ * @CLUTTER_DEVICE_REMOVED: Notification of a removed device
+ * @CLUTTER_IM_COMMIT: Input method text commit
+ * @CLUTTER_IM_DELETE: Input method text delete action
+ * @CLUTTER_IM_PREEDIT: Input method pre-edit text notification
+ * @CLUTTER_KEY_STATE: Keyboard state change
  * @CLUTTER_EVENT_LAST: Marks the end of the #ClutterEventType enumeration;
  *
  * Types of events.
@@ -562,11 +556,13 @@ typedef enum /*< prefix=CLUTTER >*/
   CLUTTER_PAD_BUTTON_RELEASE,
   CLUTTER_PAD_STRIP,
   CLUTTER_PAD_RING,
+  CLUTTER_PAD_DIAL,
   CLUTTER_DEVICE_ADDED,
   CLUTTER_DEVICE_REMOVED,
   CLUTTER_IM_COMMIT,
   CLUTTER_IM_DELETE,
   CLUTTER_IM_PREEDIT,
+  CLUTTER_KEY_STATE,
 
   CLUTTER_EVENT_LAST            /* helper */
 } ClutterEventType;
@@ -752,42 +748,20 @@ typedef enum
   CLUTTER_PICK_ALL
 } ClutterPickMode;
 
-/**
- * ClutterSwipeDirection:
- * @CLUTTER_SWIPE_DIRECTION_UP: Upwards swipe gesture
- * @CLUTTER_SWIPE_DIRECTION_DOWN: Downwards swipe gesture
- * @CLUTTER_SWIPE_DIRECTION_LEFT: Leftwards swipe gesture
- * @CLUTTER_SWIPE_DIRECTION_RIGHT: Rightwards swipe gesture
- *
- * The main direction of the swipe gesture
- */
-typedef enum /*< prefix=CLUTTER_SWIPE_DIRECTION >*/
-{
-  CLUTTER_SWIPE_DIRECTION_UP    = 1 << 0,
-  CLUTTER_SWIPE_DIRECTION_DOWN  = 1 << 1,
-  CLUTTER_SWIPE_DIRECTION_LEFT  = 1 << 2,
-  CLUTTER_SWIPE_DIRECTION_RIGHT = 1 << 3
-} ClutterSwipeDirection;
-
 /**
  * ClutterPanAxis:
- * @CLUTTER_PAN_AXIS_NONE: No constraint
- * @CLUTTER_PAN_X_AXIS: Set a constraint on the X axis
- * @CLUTTER_PAN_Y_AXIS: Set a constraint on the Y axis
- * @CLUTTER_PAN_AXIS_AUTO: Constrain panning automatically based on initial
- *   movement (available since 1.24)
+ * @CLUTTER_PAN_AXIS_BOTH: No constraint
+ * @CLUTTER_PAN_AXIS_X: Set a constraint on the X axis
+ * @CLUTTER_PAN_AXIS_Y: Set a constraint on the Y axis
  *
  * The axis of the constraint that should be applied on the
  * panning action
  */
-typedef enum /*< prefix=CLUTTER_PAN >*/
+typedef enum /*< prefix=CLUTTER_PAN_AXIS >*/
 {
-  CLUTTER_PAN_AXIS_NONE = 0,
-
-  CLUTTER_PAN_X_AXIS,
-  CLUTTER_PAN_Y_AXIS,
-
-  CLUTTER_PAN_AXIS_AUTO
+  CLUTTER_PAN_AXIS_BOTH,
+  CLUTTER_PAN_AXIS_X,
+  CLUTTER_PAN_AXIS_Y,
 } ClutterPanAxis;
 
 /**
@@ -979,28 +953,6 @@ typedef enum
   CLUTTER_STEP_MODE_END
 } ClutterStepMode;
 
-/**
- * ClutterGestureTriggerEdge:
- * @CLUTTER_GESTURE_TRIGGER_EDGE_NONE: Tell #ClutterGestureAction that
- * the gesture must begin immediately and there's no drag limit that
- * will cause its cancellation;
- * @CLUTTER_GESTURE_TRIGGER_EDGE_AFTER: Tell #ClutterGestureAction that
- * it needs to wait until the drag threshold has been exceeded before
- * considering that the gesture has begun;
- * @CLUTTER_GESTURE_TRIGGER_EDGE_BEFORE: Tell #ClutterGestureAction that
- * the gesture must begin immediately and that it must be cancelled
- * once the drag exceed the configured threshold.
- *
- * Enum passed to the [method@GestureAction.set_threshold_trigger_edge]
- * function.
- */
-typedef enum
-{
-  CLUTTER_GESTURE_TRIGGER_EDGE_NONE  = 0,
-  CLUTTER_GESTURE_TRIGGER_EDGE_AFTER,
-  CLUTTER_GESTURE_TRIGGER_EDGE_BEFORE
-} ClutterGestureTriggerEdge;
-
 /**
  * ClutterTouchpadGesturePhase:
  * @CLUTTER_TOUCHPAD_GESTURE_PHASE_BEGIN: The gesture has begun.
@@ -1051,7 +1003,7 @@ typedef enum
  * The scroll source determines the source of the scroll event.
  *
  * Keep in mind that the source device #ClutterInputDeviceType is not enough
- * to infer the scroll source.6
+ * to infer the scroll source.
  */
 typedef enum
 {
@@ -1061,6 +1013,20 @@ typedef enum
   CLUTTER_SCROLL_SOURCE_CONTINUOUS
 } ClutterScrollSource;
 
+/**
+ * ClutterScrollFlags:
+ * @CLUTTER_SCROLL_NONE: No additional information.
+ * @CLUTTER_SCROLL_INVERTED: Scroll direction is inverted, relative to the
+ *   physical motion on the device.
+ *
+ * Flags used to notify modification on scrolling behavior.
+ */
+typedef enum
+{
+  CLUTTER_SCROLL_NONE = 0,
+  CLUTTER_SCROLL_INVERTED = 1 << 0,
+} ClutterScrollFlags;
+
 /**
  * ClutterScrollFinishFlags:
  * @CLUTTER_SCROLL_FINISHED_NONE: no axis was stopped.
@@ -1069,7 +1035,7 @@ typedef enum
  *
  * Flags used to notify the axes that were stopped in a #ClutterScrollEvent.
  *
- * These can be used to trigger post-scroll effects like kinetic scrolling.6
+ * These can be used to trigger post-scroll effects like kinetic scrolling.
  */
 typedef enum
 {
@@ -1114,6 +1080,7 @@ typedef enum
   CLUTTER_PAD_FEATURE_BUTTON,
   CLUTTER_PAD_FEATURE_RING,
   CLUTTER_PAD_FEATURE_STRIP,
+  CLUTTER_PAD_FEATURE_DIAL,
 } ClutterInputDevicePadFeature;
 
 typedef enum
