@@ -28,22 +28,22 @@
  *
  */
 
+#pragma once
+
 #if !defined(__COGL_H_INSIDE__) && !defined(COGL_COMPILATION)
 #error "Only <cogl/cogl.h> can be included directly."
 #endif
 
-#ifndef __COGL_SHADER_H__
-#define __COGL_SHADER_H__
-
-#include <cogl/cogl-types.h>
-#include <cogl/cogl-defines.h>
-#include <cogl/cogl-macros.h>
+#include "cogl/cogl-types.h"
+#include "cogl/cogl-macros.h"
+#include "cogl/deprecated/cogl-program.h"
 
 G_BEGIN_DECLS
 
 /**
- * SECTION:cogl-shaders
- * @short_description: Fuctions for accessing the programmable GL pipeline
+ * CoglShader:
+ *
+ * Functions for accessing the programmable GL pipeline
  *
  * Cogl allows accessing the GL programmable pipeline in order to create
  * vertex and fragment shaders.
@@ -56,277 +56,141 @@ G_BEGIN_DECLS
  *
  * For use in all GLSL shaders, the Cogl builtins are as follows:
  *
- * <tip>
- * <glosslist>
- *  <glossentry>
- *   <glossterm>uniform mat4
- *         <emphasis>cogl_modelview_matrix</emphasis></glossterm>
- *   <glossdef><para>
+ * - `uniform mat4 cogl_modelview_matrix`
  *    The current modelview matrix. This is equivalent to
  *    #gl_ModelViewMatrix.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>uniform mat4
- *         <emphasis>cogl_projection_matrix</emphasis></glossterm>
- *   <glossdef><para>
+ * - `uniform mat4 cogl_projection_matrix`
  *    The current projection matrix. This is equivalent to
  *    #gl_ProjectionMatrix.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>uniform mat4
- *         <emphasis>cogl_modelview_projection_matrix</emphasis></glossterm>
- *   <glossdef><para>
+* - `uniform mat4 cogl_modelview_projection_matrix`
  *    The combined modelview and projection matrix. A vertex shader
  *    would typically use this to transform the incoming vertex
  *    position. The separate modelview and projection matrices are
  *    usually only needed for lighting calculations. This is
  *    equivalent to #gl_ModelViewProjectionMatrix.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>uniform mat4
- *         <emphasis>cogl_texture_matrix</emphasis>[]</glossterm>
- *   <glossdef><para>
+ * - `uniform mat4 cogl_texture_matrix[]`
  *    An array of matrices for transforming the texture
  *    coordinates. This is equivalent to #gl_TextureMatrix.
- *   </para></glossdef>
- *  </glossentry>
- * </glosslist>
- * </tip>
  *
  * In a vertex shader, the following are also available:
  *
- * <tip>
- * <glosslist>
- *  <glossentry>
- *   <glossterm>attribute vec4
- *         <emphasis>cogl_position_in</emphasis></glossterm>
- *   <glossdef><para>
+ * - `attribute vec4 cogl_position_in`
  *    The incoming vertex position. This is equivalent to #gl_Vertex.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>attribute vec4
- *         <emphasis>cogl_color_in</emphasis></glossterm>
- *   <glossdef><para>
+ * - `attribute vec4 cogl_color_in`
  *    The incoming vertex color. This is equivalent to #gl_Color.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>attribute vec4
- *         <emphasis>cogl_tex_coord_in</emphasis></glossterm>
- *   <glossdef><para>
+ * - `attribute vec4 cogl_tex_coord_in`
  *    The texture coordinate for the first texture unit. This is
  *    equivalent to #gl_MultiTexCoord0.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>attribute vec4
- *         <emphasis>cogl_tex_coord0_in</emphasis></glossterm>
- *   <glossdef><para>
+ * - `attribute vec4 cogl_tex_coord0_in`
  *    The texture coordinate for the first texture unit. This is
  *    equivalent to #gl_MultiTexCoord0. There is also
  *    #cogl_tex_coord1_in and so on.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>attribute vec3
- *         <emphasis>cogl_normal_in</emphasis></glossterm>
- *   <glossdef><para>
+ * - `attribute vec3 cogl_normal_in`
  *    The normal of the vertex. This is equivalent to #gl_Normal.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>vec4
- *         <emphasis>cogl_position_out</emphasis></glossterm>
- *   <glossdef><para>
+ * - `vec4 cogl_position_out`
  *    The calculated position of the vertex. This must be written to
  *    in all vertex shaders. This is equivalent to #gl_Position.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>float
- *         <emphasis>cogl_point_size_out</emphasis></glossterm>
- *   <glossdef><para>
+ * - `float cogl_point_size_out`
  *    The calculated size of a point. This is equivalent to #gl_PointSize.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>varying vec4
- *         <emphasis>cogl_color_out</emphasis></glossterm>
- *   <glossdef><para>
+ * - `varying vec4 cogl_color_out`
  *    The calculated color of a vertex. This is equivalent to #gl_FrontColor.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>varying vec4
- *         <emphasis>cogl_tex_coord_out</emphasis>[]</glossterm>
- *   <glossdef><para>
+ * - `varying vec4 cogl_tex_coord_out[]`
  *    An array of calculated texture coordinates for a vertex. This is
  *    equivalent to #gl_TexCoord.
- *   </para></glossdef>
- *  </glossentry>
- * </glosslist>
- * </tip>
  *
  * In a fragment shader, the following are also available:
  *
- * <tip>
- * <glosslist>
- *  <glossentry>
- *   <glossterm>varying vec4 <emphasis>cogl_color_in</emphasis></glossterm>
- *   <glossdef><para>
+ * - `varying vec4 cogl_color_in`
  *    The calculated color of a vertex. This is equivalent to #gl_FrontColor.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>varying vec4
- *              <emphasis>cogl_tex_coord_in</emphasis>[]</glossterm>
- *   <glossdef><para>
+ * - `varying vec4 cogl_tex_coord_in[]`
  *    An array of calculated texture coordinates for a vertex. This is
  *    equivalent to #gl_TexCoord.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>vec4 <emphasis>cogl_color_out</emphasis></glossterm>
- *   <glossdef><para>
+ * - `vec4 cogl_color_out`
  *    The final calculated color of the fragment. All fragment shaders
  *    must write to this variable. This is equivalent to
  *    #gl_FrontColor.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>float <emphasis>cogl_depth_out</emphasis></glossterm>
- *   <glossdef><para>
+ * - `float cogl_depth_out`
  *    An optional output variable specifying the depth value to use
  *    for this fragment. This is equivalent to #gl_FragDepth.
- *   </para></glossdef>
- *  </glossentry>
- *  <glossentry>
- *   <glossterm>bool <emphasis>cogl_front_facing</emphasis></glossterm>
- *   <glossdef><para>
+ * - `bool cogl_front_facing`
  *    A readonly variable that will be true if the current primitive
  *    is front facing. This can be used to implement two-sided
  *    coloring algorithms. This is equivalent to #gl_FrontFacing.
- *   </para></glossdef>
- *  </glossentry>
- * </glosslist>
- * </tip>
  *
  * It's worth nothing that this API isn't what Cogl would like to have
  * in the long term and it may be removed in Cogl 2.0. The
  * experimental #CoglShader API is the proposed replacement.
  */
 
-/**
- * CoglShaderType:
- * @COGL_SHADER_TYPE_VERTEX: A program for proccessing vertices
- * @COGL_SHADER_TYPE_FRAGMENT: A program for processing fragments
- *
- * Types of shaders
- *
- * Since: 1.0
- */
-typedef enum
-{
-  COGL_SHADER_TYPE_VERTEX,
-  COGL_SHADER_TYPE_FRAGMENT
-} CoglShaderType;
+#define COGL_TYPE_SHADER (cogl_shader_get_type ())
+
+COGL_EXPORT
+G_DECLARE_FINAL_TYPE (CoglShader,
+                      cogl_shader,
+                      COGL,
+                      SHADER,
+                      GObject)
 
 /**
- * cogl_create_shader:
+ * cogl_shader_new:
  * @shader_type: COGL_SHADER_TYPE_VERTEX or COGL_SHADER_TYPE_FRAGMENT.
  *
  * Create a new shader handle, use cogl_shader_source() to set the
  * source code to be used on it.
  *
- * Returns: a new shader handle.
+ * Returns: (transfer full): a new shader handle.
  * Deprecated: 1.16: Use #CoglSnippet api
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-CoglHandle
-cogl_create_shader (CoglShaderType shader_type);
-
-/**
- * cogl_is_shader:
- * @handle: A CoglHandle
- *
- * Gets whether the given handle references an existing shader object.
- *
- * Returns: %TRUE if the handle references a shader,
- *   %FALSE otherwise
- * Deprecated: 1.16: Use #CoglSnippet api
- */
-COGL_DEPRECATED_FOR (cogl_snippet_)
-gboolean
-cogl_is_shader (CoglHandle handle);
+COGL_EXPORT CoglShader*
+cogl_shader_new (CoglShaderType shader_type);
 
 /**
  * cogl_shader_source:
- * @shader: #CoglHandle for a shader.
+ * @self: A shader.
  * @source: Shader source.
  *
  * Replaces the current source associated with a shader with a new
  * one.
  *
- * Please see <link
- * linkend="cogl-Shaders-and-Programmable-Pipeline.description">above</link>
- * for a description of the recommended format for the shader code.
  * Deprecated: 1.16: Use #CoglSnippet api
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-void
-cogl_shader_source (CoglHandle  shader,
+COGL_EXPORT void
+cogl_shader_source (CoglShader *self,
                     const char *source);
 
 /**
- * cogl_shader_get_type:
- * @handle: #CoglHandle for a shader.
+ * cogl_shader_get_shader_type:
+ * @self: #CoglShader for a shader.
  *
- * Retrieves the type of a shader #CoglHandle
+ * Retrieves the type of a shader
  *
  * Return value: %COGL_SHADER_TYPE_VERTEX if the shader is a vertex processor
- *          or %COGL_SHADER_TYPE_FRAGMENT if the shader is a frament processor
+ *          or %COGL_SHADER_TYPE_FRAGMENT if the shader is a fragment processor
  * Deprecated: 1.16: Use #CoglSnippet api
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-CoglShaderType
-cogl_shader_get_type (CoglHandle handle);
+COGL_EXPORT CoglShaderType
+cogl_shader_get_shader_type (CoglShader *self);
 
 /**
- * cogl_create_program:
+ * cogl_program_new:
  *
  * Create a new cogl program object that can be used to replace parts of the GL
  * rendering pipeline with custom code.
  *
- * Returns: a new cogl program.
+ * Returns: (transfer full): a new cogl program.
  * Deprecated: 1.16: Use #CoglSnippet api
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-CoglHandle
-cogl_create_program (void);
-
-/**
- * cogl_is_program:
- * @handle: A CoglHandle
- *
- * Gets whether the given handle references an existing program object.
- *
- * Returns: %TRUE if the handle references a program,
- *   %FALSE otherwise
- *
- * Deprecated: 1.16: Use #CoglSnippet api
- */
-COGL_DEPRECATED_FOR (cogl_snippet_)
-gboolean
-cogl_is_program (CoglHandle handle);
+COGL_EXPORT CoglProgram*
+cogl_program_new (void);
 
 /**
  * cogl_program_attach_shader:
- * @program_handle: a #CoglHandle for a shdaer program.
- * @shader_handle: a #CoglHandle for a vertex of fragment shader.
+ * @program: a #CoglProgram for a shader program.
+ * @shader: a #CoglShader for a vertex of fragment shader.
  *
  * Attaches a shader to a program object. A program can have multiple
  * vertex or fragment shaders but only one of them may provide a
@@ -336,13 +200,13 @@ cogl_is_program (CoglHandle handle);
  * Deprecated: 1.16: Use #CoglSnippet api
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-void
-cogl_program_attach_shader (CoglHandle program_handle,
-                            CoglHandle shader_handle);
+COGL_EXPORT void
+cogl_program_attach_shader (CoglProgram *program,
+                            CoglShader  *shader);
 
 /**
  * cogl_program_link:
- * @handle: a #CoglHandle for a shader program.
+ * @program: A shader program.
  *
  * Links a program making it ready for use. Note that calling this
  * function is optional. If it is not called the program will
@@ -351,12 +215,12 @@ cogl_program_attach_shader (CoglHandle program_handle,
  * Deprecated: 1.16: Use #CoglSnippet api
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-void
-cogl_program_link (CoglHandle handle);
+COGL_EXPORT void
+cogl_program_link (CoglProgram *program);
 
 /**
  * cogl_program_get_uniform_location:
- * @handle: a #CoglHandle for a shader program.
+ * @program: A shader program.
  * @uniform_name: the name of a uniform.
  *
  * Retrieve the location (offset) of a uniform variable in a shader program,
@@ -367,53 +231,49 @@ cogl_program_link (CoglHandle handle);
  * Deprecated: 1.16: Use #CoglSnippet api instead
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-int
-cogl_program_get_uniform_location (CoglHandle  handle,
-                                   const char *uniform_name);
+COGL_EXPORT int
+cogl_program_get_uniform_location (CoglProgram *program,
+                                   const char  *uniform_name);
 
 /**
  * cogl_program_set_uniform_1f:
- * @program: A #CoglHandle for a linked program
+ * @program: A linked program
  * @uniform_location: the uniform location retrieved from
- *    cogl_program_get_uniform_location().
+ *    [method@Program.get_uniform_location].
  * @value: the new value of the uniform.
  *
  * Changes the value of a floating point uniform for the given linked
  * @program.
- *
- * Since: 1.4
  * Deprecated: 1.16: Use #CoglSnippet api instead
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-void
-cogl_program_set_uniform_1f (CoglHandle program,
-                             int uniform_location,
-                             float value);
+COGL_EXPORT void
+cogl_program_set_uniform_1f (CoglProgram *program,
+                             int          uniform_location,
+                             float        value);
 
 /**
  * cogl_program_set_uniform_1i:
- * @program: A #CoglHandle for a linked program
+ * @program: A linked program
  * @uniform_location: the uniform location retrieved from
- *    cogl_program_get_uniform_location().
+ *    [method@Program.get_uniform_location].
  * @value: the new value of the uniform.
  *
  * Changes the value of an integer uniform for the given linked
  * @program.
- *
- * Since: 1.4
  * Deprecated: 1.16: Use #CoglSnippet api instead
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-void
-cogl_program_set_uniform_1i (CoglHandle program,
-                             int uniform_location,
-                             int value);
+COGL_EXPORT void
+cogl_program_set_uniform_1i (CoglProgram *program,
+                             int          uniform_location,
+                             int          value);
 
 /**
  * cogl_program_set_uniform_float:
- * @program: A #CoglHandle for a linked program
+ * @program: A linked program
  * @uniform_location: the uniform location retrieved from
- *    cogl_program_get_uniform_location().
+ *    [method@Program.get_uniform_location].
  * @n_components: The number of components for the uniform. For
  * example with glsl you'd use 3 for a vec3 or 4 for a vec4.
  * @count: For uniform arrays this is the array length otherwise just
@@ -422,23 +282,21 @@ cogl_program_set_uniform_1i (CoglHandle program,
  *
  * Changes the value of a float vector uniform, or uniform array for
  * the given linked @program.
- *
- * Since: 1.4
  * Deprecated: 1.16: Use #CoglSnippet api instead
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-void
-cogl_program_set_uniform_float (CoglHandle program,
-                                int uniform_location,
-                                int n_components,
-                                int count,
+COGL_EXPORT void
+cogl_program_set_uniform_float (CoglProgram *program,
+                                int          uniform_location,
+                                int          n_components,
+                                int          count,
                                 const float *value);
 
 /**
  * cogl_program_set_uniform_int:
- * @program: A #CoglHandle for a linked program
+ * @program: A linked program
  * @uniform_location: the uniform location retrieved from
- *    cogl_program_get_uniform_location().
+ *    [method@Program.get_uniform_location].
  * @n_components: The number of components for the uniform. For
  * example with glsl you'd use 3 for a vec3 or 4 for a vec4.
  * @count: For uniform arrays this is the array length otherwise just
@@ -447,23 +305,21 @@ cogl_program_set_uniform_float (CoglHandle program,
  *
  * Changes the value of a int vector uniform, or uniform array for
  * the given linked @program.
- *
- * Since: 1.4
  * Deprecated: 1.16: Use #CoglSnippet api instead
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-void
-cogl_program_set_uniform_int (CoglHandle program,
-                              int uniform_location,
-                              int n_components,
-                              int count,
-                              const int *value);
+COGL_EXPORT void
+cogl_program_set_uniform_int (CoglProgram *program,
+                              int          uniform_location,
+                              int          n_components,
+                              int          count,
+                              const int   *value);
 
 /**
  * cogl_program_set_uniform_matrix:
- * @program: A #CoglHandle for a linked program
+ * @program: A linked program
  * @uniform_location: the uniform location retrieved from
- *    cogl_program_get_uniform_location().
+ *    [method@Program.get_uniform_location].
  * @dimensions: The dimensions of the matrix. So for for example pass
  *    2 for a 2x2 matrix or 3 for 3x3.
  * @count: For uniform arrays this is the array length otherwise just
@@ -473,19 +329,15 @@ cogl_program_set_uniform_int (CoglHandle program,
  *
  * Changes the value of a matrix uniform, or uniform array in the
  * given linked @program.
- *
- * Since: 1.4
  * Deprecated: 1.16: Use #CoglSnippet api instead
  */
 COGL_DEPRECATED_FOR (cogl_snippet_)
-void
-cogl_program_set_uniform_matrix (CoglHandle program,
-                                 int uniform_location,
-                                 int dimensions,
-                                 int count,
-                                 gboolean transpose,
+COGL_EXPORT void
+cogl_program_set_uniform_matrix (CoglProgram *program,
+                                 int          uniform_location,
+                                 int          dimensions,
+                                 int          count,
+                                 gboolean     transpose,
                                  const float *value);
 
 G_END_DECLS
-
-#endif /* __COGL_SHADER_H__ */
