@@ -113,24 +113,17 @@ _cogl_winsys_renderer_get_proc_address (CoglRenderer *renderer,
   GCallback result = eglGetProcAddress (name);
 
   if (result == NULL)
-    g_module_symbol (renderer->libgl_module, name, (gpointer *)&result);
+    g_module_symbol (cogl_renderer_get_gl_module (renderer), name, (gpointer *)&result);
 
   return result;
 }
 
-static void
-_cogl_winsys_renderer_disconnect (CoglRenderer *renderer)
-{
-  /* This function must be overridden by a platform winsys */
-  g_assert_not_reached ();
-}
-
 static void
 _cogl_winsys_renderer_bind_api (CoglRenderer *renderer)
 {
-  if (renderer->driver_id == COGL_DRIVER_ID_GL3)
+  if (cogl_renderer_get_driver_id (renderer) == COGL_DRIVER_ID_GL3)
     eglBindAPI (EGL_OPENGL_API);
-  else if (renderer->driver_id == COGL_DRIVER_ID_GLES2)
+  else if (cogl_renderer_get_driver_id (renderer) == COGL_DRIVER_ID_GLES2)
     eglBindAPI (EGL_OPENGL_ES_API);
 }
 
@@ -138,7 +131,7 @@ _cogl_winsys_renderer_bind_api (CoglRenderer *renderer)
 static void
 check_egl_extensions (CoglRenderer *renderer)
 {
-  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (renderer);
   const char *egl_extensions;
   char **split_extensions;
   int i;
@@ -167,7 +160,7 @@ gboolean
 _cogl_winsys_egl_renderer_connect_common (CoglRenderer *renderer,
                                           GError **error)
 {
-  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (renderer);
 
   if (!eglInitialize (egl_renderer->edpy,
                       &egl_renderer->egl_version_major,
@@ -198,7 +191,7 @@ cogl_display_egl_determine_attributes (CoglDisplay *display,
                                        EGLint      *attributes)
 {
   CoglRenderer *renderer = display->renderer;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (renderer);
   int i = 0;
 
   /* Let the platform add attributes first, including setting the
@@ -226,7 +219,7 @@ cogl_display_egl_determine_attributes (CoglDisplay *display,
   attributes[i++] = EGL_DONT_CARE;
 
   attributes[i++] = EGL_RENDERABLE_TYPE;
-  attributes[i++] = (renderer->driver_id == COGL_DRIVER_ID_GL3 ?
+  attributes[i++] = (cogl_renderer_get_driver_id (renderer) == COGL_DRIVER_ID_GL3 ?
                      EGL_OPENGL_BIT :
                      EGL_OPENGL_ES2_BIT);
 
@@ -242,7 +235,7 @@ _cogl_winsys_egl_make_current (CoglDisplay *display,
                                EGLContext context)
 {
   CoglDisplayEGL *egl_display = display->winsys;
-  CoglRendererEGL *egl_renderer = display->renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (display->renderer);
   EGLBoolean ret;
 
   if (egl_display->current_draw_surface == draw &&
@@ -266,7 +259,7 @@ EGLBoolean
 _cogl_winsys_egl_ensure_current (CoglDisplay *display)
 {
   CoglDisplayEGL *egl_display = display->winsys;
-  CoglRendererEGL *egl_renderer = display->renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (display->renderer);
 
   return eglMakeCurrent (egl_renderer->edpy,
                          egl_display->current_draw_surface,
@@ -279,7 +272,7 @@ cleanup_context (CoglDisplay *display)
 {
   CoglRenderer *renderer = display->renderer;
   CoglDisplayEGL *egl_display = display->winsys;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (renderer);
 
   if (egl_display->egl_context != EGL_NO_CONTEXT)
     {
@@ -300,7 +293,7 @@ try_create_context (CoglDisplay *display,
 {
   CoglRenderer *renderer = display->renderer;
   CoglDisplayEGL *egl_display = display->winsys;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (renderer);
   EGLDisplay edpy;
   EGLConfig config;
   EGLint attribs[11];
@@ -337,7 +330,7 @@ try_create_context (CoglDisplay *display,
       egl_display->egl_config = config;
     }
 
-  if (display->renderer->driver_id == COGL_DRIVER_ID_GL3)
+  if (cogl_renderer_get_driver_id (display->renderer) == COGL_DRIVER_ID_GL3)
     {
       if (!(egl_renderer->private_features &
             COGL_EGL_WINSYS_FEATURE_CREATE_CONTEXT))
@@ -356,7 +349,7 @@ try_create_context (CoglDisplay *display,
       attribs[i++] = EGL_CONTEXT_OPENGL_PROFILE_MASK_KHR;
       attribs[i++] = EGL_CONTEXT_OPENGL_CORE_PROFILE_BIT_KHR;
     }
-  else if (display->renderer->driver_id == COGL_DRIVER_ID_GLES2)
+  else if (cogl_renderer_get_driver_id (display->renderer) == COGL_DRIVER_ID_GLES2)
     {
       attribs[i++] = EGL_CONTEXT_CLIENT_VERSION;
       attribs[i++] = 2;
@@ -429,7 +422,7 @@ err:
 static void
 _cogl_winsys_display_destroy (CoglDisplay *display)
 {
-  CoglRendererEGL *egl_renderer = display->renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (display->renderer);
   CoglDisplayEGL *egl_display = display->winsys;
 
   g_return_if_fail (egl_display != NULL);
@@ -452,7 +445,7 @@ _cogl_winsys_display_setup (CoglDisplay *display,
 {
   CoglDisplayEGL *egl_display;
   CoglRenderer *renderer = display->renderer;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (renderer);
 
   g_return_val_if_fail (display->winsys == NULL, FALSE);
 
@@ -478,7 +471,7 @@ _cogl_winsys_context_init (CoglContext *context, GError **error)
 {
   CoglRenderer *renderer = context->display->renderer;
   CoglDisplayEGL *egl_display = context->display->winsys;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (renderer);
 
   context->winsys = g_new0 (CoglContextEGL, 1);
 
@@ -525,7 +518,7 @@ static void
 _cogl_winsys_context_deinit (CoglContext *context)
 {
   CoglRenderer *renderer = context->display->renderer;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (renderer);
 
   if (egl_renderer->platform_vtable->context_deinit)
     egl_renderer->platform_vtable->context_deinit (context);
@@ -538,7 +531,7 @@ _cogl_winsys_context_deinit (CoglContext *context)
 static int
 _cogl_winsys_get_sync_fd (CoglContext *context)
 {
-  CoglRendererEGL *renderer = context->display->renderer->winsys;
+  CoglRendererEGL *renderer = cogl_renderer_get_winsys (context->display->renderer);
   int fd;
 
   if (!renderer->pf_eglDupNativeFenceFD)
@@ -554,7 +547,7 @@ _cogl_winsys_get_sync_fd (CoglContext *context)
 static void
 _cogl_winsys_update_sync (CoglContext *context)
 {
-  CoglRendererEGL *renderer = context->display->renderer->winsys;
+  CoglRendererEGL *renderer = cogl_renderer_get_winsys (context->display->renderer);
 
   if (!renderer->pf_eglDestroySync || !renderer->pf_eglCreateSync)
     return;
@@ -576,7 +569,6 @@ static CoglWinsysVtable _cogl_winsys_vtable =
 
     .renderer_get_proc_address = _cogl_winsys_renderer_get_proc_address,
     .renderer_connect = _cogl_winsys_renderer_connect,
-    .renderer_disconnect = _cogl_winsys_renderer_disconnect,
     .renderer_bind_api = _cogl_winsys_renderer_bind_api,
     .display_setup = _cogl_winsys_display_setup,
     .display_destroy = _cogl_winsys_display_destroy,
@@ -612,7 +604,7 @@ _cogl_egl_create_image (CoglContext *ctx,
                         const EGLint *attribs)
 {
   CoglDisplayEGL *egl_display = ctx->display->winsys;
-  CoglRendererEGL *egl_renderer = ctx->display->renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (ctx->display->renderer);
   EGLContext egl_ctx;
 
   g_return_val_if_fail (egl_renderer->pf_eglCreateImage, EGL_NO_IMAGE_KHR);
@@ -644,7 +636,7 @@ void
 _cogl_egl_destroy_image (CoglContext *ctx,
                          EGLImageKHR image)
 {
-  CoglRendererEGL *egl_renderer = ctx->display->renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (ctx->display->renderer);
 
   g_return_if_fail (egl_renderer->pf_eglDestroyImage);
 
@@ -655,7 +647,7 @@ _cogl_egl_destroy_image (CoglContext *ctx,
 EGLDisplay
 cogl_context_get_egl_display (CoglContext *context)
 {
-  CoglRendererEGL *egl_renderer = context->display->renderer->winsys;
+  CoglRendererEGL *egl_renderer = cogl_renderer_get_winsys (context->display->renderer);
 
   return egl_renderer->edpy;
 }
