@@ -14,16 +14,13 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jonas Ådahl <jadahl@gmail.com>
  */
 
-#ifndef META_STAGE_X11_NESTED_H
-#define META_STAGE_X11_NESTED_H
+#pragma once
 
 #include "clutter/clutter-mutter.h"
 #include "backends/x11/meta-stage-x11.h"
@@ -31,5 +28,3 @@
 #define META_TYPE_STAGE_X11_NESTED (meta_stage_x11_nested_get_type ())
 G_DECLARE_FINAL_TYPE (MetaStageX11Nested, meta_stage_x11_nested,
                       META, STAGE_X11_NESTED, MetaStageX11)
-
-#endif /* META_STAGE_X11_NESTED_H */
