@@ -177,8 +177,9 @@ update_pixmap (MetaSurfaceActorX11 *self)
 
       if (new_pixmap == None)
         {
-          meta_verbose ("Unable to get named pixmap for %s",
-                        meta_window_get_description (self->window));
+          meta_topic (META_DEBUG_RENDER,
+                      "Unable to get named pixmap for %s",
+                      meta_window_get_description (self->window));
           return;
         }
 
