@@ -185,11 +185,12 @@ queue_send_frame_messages_timeout (MetaWindowActorX11 *actor_x11)
 }
 
 static void
-assign_frame_counter_to_frames (MetaWindowActorX11 *actor_x11,
-                                ClutterFrame       *clutter_frame)
+assign_frame_counter_to_frames (MetaWindowActorX11 *actor_x11)
 {
   MetaWindow *window =
     meta_window_actor_get_meta_window (META_WINDOW_ACTOR (actor_x11));
+  MetaCompositor *compositor = window->display->compositor;
+  ClutterStage *stage = meta_compositor_get_stage (compositor);
   MetaFrame *frame;
   MetaSyncCounter *sync_counter;
 
@@ -201,13 +202,13 @@ assign_frame_counter_to_frames (MetaWindowActorX11 *actor_x11,
 
   sync_counter = meta_window_x11_get_sync_counter (window);
   meta_sync_counter_assign_counter_to_frames (sync_counter,
-                                              clutter_frame->frame_count);
+                                              clutter_stage_get_frame_counter (stage));
   frame = meta_window_x11_get_frame (window);
   if (frame)
     {
       sync_counter = meta_frame_get_sync_counter (frame);
       meta_sync_counter_assign_counter_to_frames (sync_counter,
-                                                  clutter_frame->frame_count);
+                                                  clutter_stage_get_frame_counter (stage));
     }
 }
 
@@ -1218,7 +1219,7 @@ meta_window_actor_x11_before_paint (MetaWindowActor  *actor,
 
   handle_updates (actor_x11);
 
-  assign_frame_counter_to_frames (actor_x11, frame);
+  assign_frame_counter_to_frames (actor_x11);
 }
 
 static void
@@ -1236,12 +1237,8 @@ meta_window_actor_x11_paint (ClutterActor        *actor,
   * and send the completion events normally */
   if (actor_x11->send_frame_messages_timer != 0)
     {
-      ClutterFrame *frame;
-
       remove_frame_messages_timer (actor_x11);
-      frame = clutter_paint_context_get_frame (paint_context);
-      if (frame)
-        assign_frame_counter_to_frames (actor_x11, frame);
+      assign_frame_counter_to_frames (actor_x11);
     }
 
   window = meta_window_actor_get_meta_window (META_WINDOW_ACTOR (actor_x11));
