@@ -34,11 +34,6 @@
 #include "core/window-private.h"
 #include "meta/meta-backend.h"
 
-#ifdef HAVE_X11_CLIENT
-#include "backends/x11/meta-backend-x11.h"
-#include "backends/x11/meta-input-device-x11.h"
-#endif
-
 #ifdef HAVE_NATIVE_BACKEND
 #include "backends/native/meta-backend-native.h"
 #endif
@@ -47,23 +42,9 @@
 #include "wayland/meta-wayland-private.h"
 #endif
 
-#define IS_GESTURE_EVENT(et) ((et) == CLUTTER_TOUCHPAD_SWIPE || \
-                              (et) == CLUTTER_TOUCHPAD_PINCH || \
-                              (et) == CLUTTER_TOUCHPAD_HOLD || \
-                              (et) == CLUTTER_TOUCH_BEGIN || \
-                              (et) == CLUTTER_TOUCH_UPDATE || \
-                              (et) == CLUTTER_TOUCH_END || \
-                              (et) == CLUTTER_TOUCH_CANCEL)
-
 #define IS_KEY_EVENT(et) ((et) == CLUTTER_KEY_PRESS || \
                           (et) == CLUTTER_KEY_RELEASE)
 
-typedef enum
-{
-  EVENTS_UNFREEZE_SYNC,
-  EVENTS_UNFREEZE_REPLAY,
-} EventsUnfreezeMethod;
-
 static ClutterStage *
 stage_from_display (MetaDisplay *display)
 {
@@ -123,9 +104,6 @@ handle_idletime_for_event (MetaDisplay        *display,
   ClutterEventType event_type;
   ClutterEventFlags flags;
 
-  if (clutter_event_get_device (event) == NULL)
-    return;
-
   flags = clutter_event_get_flags (event);
   event_type = clutter_event_type (event);
 
@@ -179,51 +157,6 @@ sequence_is_pointer_emulated (MetaDisplay        *display,
   return FALSE;
 }
 
-#ifdef HAVE_X11
-static void
-maybe_unfreeze_pointer_events (MetaBackend          *backend,
-                               const ClutterEvent   *event,
-                               EventsUnfreezeMethod  unfreeze_method)
-{
-  ClutterInputDevice *device;
-  Display *xdisplay;
-  int event_mode;
-  int device_id;
-  uint32_t time_ms;
-
-  if (clutter_event_type (event) != CLUTTER_BUTTON_PRESS)
-    return;
-
-  if (!META_IS_BACKEND_X11 (backend))
-    return;
-
-  device = clutter_event_get_device (event);
-  device_id = meta_input_device_x11_get_device_id (device);
-  time_ms = clutter_event_get_time (event);
-  switch (unfreeze_method)
-    {
-    case EVENTS_UNFREEZE_SYNC:
-      event_mode = XISyncDevice;
-      meta_topic (META_DEBUG_X11,
-                  "Syncing events time %u device %i",
-                  (unsigned int) time_ms, device_id);
-      break;
-    case EVENTS_UNFREEZE_REPLAY:
-      event_mode = XIReplayDevice;
-      meta_topic (META_DEBUG_X11,
-                  "Replaying events time %u device %i",
-                  (unsigned int) time_ms, device_id);
-      break;
-    default:
-      g_assert_not_reached ();
-      return;
-    }
-
-  xdisplay = meta_backend_x11_get_xdisplay (META_BACKEND_X11 (backend));
-  XIAllowEvents (xdisplay, device_id, event_mode, time_ms);
-}
-#endif
-
 static gboolean
 meta_display_handle_event (MetaDisplay        *display,
                            const ClutterEvent *event,
@@ -233,14 +166,16 @@ meta_display_handle_event (MetaDisplay        *display,
   MetaBackend *backend = meta_context_get_backend (context);
   MetaA11yManager *a11y_manager = meta_backend_get_a11y_manager (backend);
   MetaCompositor *compositor = meta_display_get_compositor (display);
-  ClutterInputDevice *device;
+  ClutterInputDevice *source_device;
   MetaWindow *window = NULL;
   MetaGestureTracker *gesture_tracker;
   ClutterEventSequence *sequence;
   ClutterEventType event_type;
+  ClutterSeat *seat;
   gboolean has_grab;
   gboolean a11y_grabbed;
   MetaTabletActionMapper *mapper;
+  MetaEventMode mode_hint;
 #ifdef HAVE_WAYLAND
   MetaWaylandCompositor *wayland_compositor;
   MetaWaylandTextInput *wayland_text_input = NULL;
@@ -275,8 +210,9 @@ meta_display_handle_event (MetaDisplay        *display,
         return CLUTTER_EVENT_STOP;
     }
 
-  device = clutter_event_get_device (event);
-  clutter_input_pointer_a11y_update (device, event);
+  source_device = clutter_event_get_source_device (event);
+  seat = clutter_input_device_get_seat (clutter_event_get_source_device (event));
+  clutter_seat_a11y_update (seat, event);
 
   /* Set the pointer emulating sequence on touch begin, if eligible */
   if (event_type == CLUTTER_TOUCH_BEGIN)
@@ -312,7 +248,8 @@ meta_display_handle_event (MetaDisplay        *display,
   if (event_type == CLUTTER_PAD_BUTTON_PRESS ||
       event_type == CLUTTER_PAD_BUTTON_RELEASE ||
       event_type == CLUTTER_PAD_RING ||
-      event_type == CLUTTER_PAD_STRIP)
+      event_type == CLUTTER_PAD_STRIP ||
+      event_type == CLUTTER_PAD_DIAL)
     {
       gboolean handle_pad_event;
       gboolean is_mode_switch = FALSE;
@@ -341,7 +278,8 @@ meta_display_handle_event (MetaDisplay        *display,
            event_type == CLUTTER_BUTTON_RELEASE)
     {
       mapper = META_TABLET_ACTION_MAPPER (display->tool_action_mapper);
-      if (((clutter_input_device_get_capabilities (device) & CLUTTER_INPUT_CAPABILITY_TABLET_TOOL) &&
+      if ((!!(clutter_input_device_get_capabilities (source_device) &
+              CLUTTER_INPUT_CAPABILITY_TABLET_TOOL) &&
            meta_tablet_action_mapper_handle_event (mapper, event)) ||
           clutter_event_get_button (event) == 0)
         return CLUTTER_EVENT_STOP;
@@ -358,26 +296,14 @@ meta_display_handle_event (MetaDisplay        *display,
       meta_tablet_action_mapper_handle_event (mapper, event);
     }
 
-  if (event_type == CLUTTER_MOTION)
+  if (event_type == CLUTTER_MOTION &&
+      !(clutter_input_device_get_capabilities (source_device) &
+        CLUTTER_INPUT_CAPABILITY_TABLET_TOOL))
     {
-#ifdef HAVE_WAYLAND
-      if (wayland_compositor)
-        {
-          MetaCursorRenderer *cursor_renderer =
-            meta_backend_get_cursor_renderer_for_device (backend, device);
+      MetaCursorTracker *cursor_tracker =
+        meta_backend_get_cursor_tracker (backend);
 
-          if (cursor_renderer)
-            meta_cursor_renderer_update_position (cursor_renderer);
-        }
-#endif
-
-      if (device == clutter_seat_get_pointer (clutter_input_device_get_seat (device)))
-        {
-          MetaCursorTracker *cursor_tracker =
-            meta_backend_get_cursor_tracker (backend);
-
-          meta_cursor_tracker_invalidate_position (cursor_tracker);
-        }
+      meta_cursor_tracker_invalidate_position (cursor_tracker);
     }
 
   window = get_window_for_event (display, event, event_actor);
@@ -437,7 +363,7 @@ meta_display_handle_event (MetaDisplay        *display,
       ClutterModifierType grab_mods;
 
       grab_mods = meta_display_get_compositor_modifiers (display);
-      if ((clutter_event_get_state (event) & grab_mods) != 0)
+      if ((clutter_event_get_state (event) & grab_mods) == grab_mods)
         return CLUTTER_EVENT_PROPAGATE;
     }
 
@@ -452,10 +378,6 @@ meta_display_handle_event (MetaDisplay        *display,
       if (meta_window_handle_ungrabbed_event (window, event))
         return CLUTTER_EVENT_STOP;
 
-#ifdef HAVE_X11
-      /* Now replay the button press event to release our own sync grab. */
-      maybe_unfreeze_pointer_events (backend, event, EVENTS_UNFREEZE_REPLAY);
-#endif
       /* If the focus window has an active close dialog let clutter
        * events go through, so fancy clutter dialogs can get to handle
        * all events.
@@ -463,15 +385,16 @@ meta_display_handle_event (MetaDisplay        *display,
       if (window->close_dialog &&
           meta_close_dialog_is_visible (window->close_dialog))
         return CLUTTER_EVENT_PROPAGATE;
+
+      /* Now replay the button press event to release our own sync grab. */
+      mode_hint = META_EVENT_MODE_REPLAY;
     }
   else
     {
       /* We could not match the event with a window, make sure we sync
        * the pointer to discard the sequence and don't keep events frozen.
        */
-#ifdef HAVE_X11
-      maybe_unfreeze_pointer_events (backend, event, EVENTS_UNFREEZE_SYNC);
-#endif
+      mode_hint = META_EVENT_MODE_KEEP_FROZEN;
     }
 
 #ifdef HAVE_WAYLAND
@@ -487,14 +410,9 @@ meta_display_handle_event (MetaDisplay        *display,
       if (meta_wayland_compositor_handle_event (wayland_compositor, event))
         return CLUTTER_EVENT_STOP;
     }
-  else
 #endif
-    {
-      if (window && !IS_GESTURE_EVENT (event_type))
-        return CLUTTER_EVENT_STOP;
-    }
 
-  return CLUTTER_EVENT_PROPAGATE;
+  return meta_compositor_handle_event (compositor, event, window, mode_hint);
 }
 
 static gboolean
