@@ -12,14 +12,11 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifndef META_PRIVATE_ENUMS_H
-#define META_PRIVATE_ENUMS_H
+#pragma once
 
 typedef enum _MetaX11DisplayPolicy
 {
@@ -33,5 +30,3 @@ typedef enum
   META_QUEUE_CALC_SHOWING = 1 << 0,
   META_QUEUE_MOVE_RESIZE = 1 << 1,
 } MetaQueueType;
-
-#endif /* META_PRIVATE_ENUMS_H */
