@@ -34,11 +34,13 @@
 const char *client_id = "0";
 static gboolean wayland;
 static gboolean dont_exit_on_eof;
+static gboolean verbose;
 GHashTable *windows;
 GQuark event_source_quark;
 GQuark event_handlers_quark;
 GQuark can_take_focus_quark;
 gboolean sync_after_lines = -1;
+gboolean is_sleeping;
 
 typedef void (*XEventHandler) (GtkWidget *window, XEvent *event);
 
@@ -328,19 +330,28 @@ prepare_popup_window (GdkSeat   *seat,
   gtk_widget_show (popup);
 }
 
+typedef enum _PopupAtFlags
+{
+  POPUP_AT_FLAG_NONE = 0,
+  POPUP_AT_FLAG_GRAB = 1 << 0,
+  POPUP_AT_FLAG_RESIZE = 1 << 1,
+  POPUP_AT_FLAG_FLIP = 1 << 2,
+} PopupAtFlags;
+
 static void
-popup_at (GtkWidget  *parent,
-          const char *popup_id,
-          const char *position,
-          int         width,
-          int         height,
-          gboolean    grab)
+popup_at (GtkWidget    *parent,
+          const char   *popup_id,
+          const char   *position,
+          int           width,
+          int           height,
+          PopupAtFlags  flags)
 {
   GtkWidget *popup;
   g_autofree char *title;
   GdkWindow *gdk_window;
   GdkRectangle window_rect;
   GdkGravity rect_anchor, window_anchor;
+  GdkAnchorHints anchor_hints = 0;
 
   popup = g_object_new (GTK_TYPE_WINDOW,
                         "type", GTK_WINDOW_POPUP,
@@ -360,13 +371,20 @@ popup_at (GtkWidget  *parent,
   gtk_widget_get_allocation (popup, &window_rect);
 
   calculate_anchors (position, &rect_anchor, &window_anchor);
+
+  if (flags & POPUP_AT_FLAG_RESIZE)
+    anchor_hints |= GDK_ANCHOR_RESIZE;
+  if (flags & POPUP_AT_FLAG_FLIP)
+    anchor_hints |= GDK_ANCHOR_FLIP;
+
   gdk_window_move_to_rect (gdk_window,
                            &window_rect,
                            rect_anchor,
                            window_anchor,
-                           0, 0, 0);
+                           anchor_hints,
+                           0, 0);
 
-  if (grab)
+  if (flags & POPUP_AT_FLAG_GRAB)
     {
       GdkSeat *seat =
         gdk_display_get_default_seat (gtk_widget_get_display (popup));
@@ -409,12 +427,25 @@ find_monitor_from_connector (const char *connector)
 }
 
 static void
-process_line (const char *line)
+sleep_timeout_cb (gpointer user_data)
+{
+  GDataInputStream *in = G_DATA_INPUT_STREAM (user_data);
+
+  is_sleeping = FALSE;
+  read_next_line (in);
+}
+
+static void
+process_line (const char       *line,
+              GDataInputStream *in)
 {
   GdkDisplay *display = gdk_display_get_default ();
   GError *error = NULL;
   int argc;
   char **argv;
+  static int line_count = 0;
+
+  line_count++;
 
   if (!g_shell_parse_argv (line, &argc, &argv, &error))
     {
@@ -429,6 +460,9 @@ process_line (const char *line)
       goto out;
     }
 
+  if (verbose)
+    g_printerr ("%d %s\n", line_count, line);
+
   if (strcmp (argv[0], "create") == 0)
     {
       int i;
@@ -1165,12 +1199,14 @@ process_line (const char *line)
     {
       GtkWidget *parent;
       int width, height;
-      gboolean grab;
+      PopupAtFlags flags = POPUP_AT_FLAG_NONE;
+      int i;
 
-      if (argc != 6 &&
-          argc != 7)
+      if (argc < 6)
         {
-          g_print ("usage: popup <popup-id> <parent-id> <top|bottom|left|right|center> <width> <height> [grab]\n");
+          g_print ("usage: popup_at <popup-id> <parent-id> "
+                   "<top|bottom|left|right|center> "
+                   "<width> <height> [<grab>,<resize>,<flip>]\n");
           goto out;
         }
 
@@ -1184,11 +1220,19 @@ process_line (const char *line)
       width = atoi (argv[4]);
       height = atoi (argv[5]);
 
-      if (argc == 7)
+      for (i = 6; i < argc; i++)
         {
-          if (g_strcmp0 (argv[6], "grab") == 0)
+          if (g_strcmp0 (argv[i], "grab") == 0)
             {
-              grab = TRUE;
+              flags |= POPUP_AT_FLAG_GRAB;
+            }
+          else if (g_strcmp0 (argv[i], "resize") == 0)
+            {
+              flags |= POPUP_AT_FLAG_RESIZE;
+            }
+          else if (g_strcmp0 (argv[i], "flip") == 0)
+            {
+              flags |= POPUP_AT_FLAG_FLIP;
             }
           else
             {
@@ -1196,12 +1240,8 @@ process_line (const char *line)
               goto out;
             }
         }
-      else
-        {
-          grab = FALSE;
-        }
 
-      popup_at (parent, argv[1], argv[3], width, height, grab);
+      popup_at (parent, argv[1], argv[3], width, height, flags);
     }
   else if (strcmp (argv[0], "popup") == 0)
     {
@@ -1220,7 +1260,7 @@ process_line (const char *line)
           goto out;
         }
 
-      popup_at (parent, argv[1], "center", 100, 100, FALSE);
+      popup_at (parent, argv[1], "center", 100, 100, POPUP_AT_FLAG_NONE);
     }
   else if (strcmp (argv[0], "dismiss") == 0)
     {
@@ -1239,6 +1279,20 @@ process_line (const char *line)
       g_hash_table_remove (windows, argv[1]);
       gtk_widget_destroy (popup);
     }
+  else if (strcmp (argv[0], "sleep") == 0)
+    {
+      int64_t sleep_ms;
+
+      if (argc != 2)
+        {
+          g_print ("usage: sleep <milliseconds>\n");
+          goto out;
+        }
+
+      sleep_ms = atoi (argv[1]);
+      is_sleeping = TRUE;
+      g_timeout_add_once (sleep_ms, sleep_timeout_cb, in);
+    }
   else
     {
       g_print ("Unknown command %s\n", argv[0]);
@@ -1251,6 +1305,13 @@ process_line (const char *line)
   g_strfreev (argv);
 }
 
+static void
+maybe_read_next_line (GDataInputStream *in)
+{
+  if (!is_sleeping)
+    read_next_line (in);
+}
+
 static void
 on_line_received (GObject      *source,
                   GAsyncResult *result,
@@ -1270,9 +1331,9 @@ on_line_received (GObject      *source,
       return;
     }
 
-  process_line (line);
+  process_line (line, in);
   g_free (line);
-  read_next_line (in);
+  maybe_read_next_line (in);
 }
 
 static void
@@ -1297,7 +1358,9 @@ read_next_line (GDataInputStream *in)
           return;
         }
 
-      process_line (line);
+      process_line (line, in);
+      if (is_sleeping)
+        return;
     }
 
   if (sync_after_lines >= 0)
@@ -1326,6 +1389,12 @@ const GOptionEntry options[] = {
     "Identifier used in Window titles for this client",
     "CLIENT_ID",
   },
+  {
+    "verbose", 'v', 0, G_OPTION_ARG_NONE,
+    &verbose,
+    "Verbose",
+    NULL,
+  },
   { NULL }
 };
 
