@@ -18,14 +18,13 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef META_WAYLAND_POPUP_H
-#define META_WAYLAND_POPUP_H
+#pragma once
 
 #include <glib.h>
 #include <wayland-server.h>
 
-#include "meta-wayland-types.h"
-#include "meta-wayland-pointer.h"
+#include "wayland/meta-wayland-types.h"
+#include "wayland/meta-wayland-pointer.h"
 
 #define META_TYPE_WAYLAND_POPUP_SURFACE (meta_wayland_popup_surface_get_type ())
 G_DECLARE_INTERFACE (MetaWaylandPopupSurface, meta_wayland_popup_surface,
@@ -38,17 +37,18 @@ struct _MetaWaylandPopupSurfaceInterface
 
   void (*done) (MetaWaylandPopupSurface *popup_surface);
   void (*dismiss) (MetaWaylandPopupSurface *popup_surface);
+  gboolean (*finish) (MetaWaylandPopupSurface *popup_surface);
   MetaWaylandSurface *(*get_surface) (MetaWaylandPopupSurface *popup_surface);
 };
 
-MetaWaylandPopupGrab *meta_wayland_popup_grab_create (MetaWaylandPointer      *pointer,
+MetaWaylandPopupGrab *meta_wayland_popup_grab_create (MetaWaylandSeat         *seat,
                                                       MetaWaylandPopupSurface *popup_surface);
 
 void meta_wayland_popup_grab_destroy (MetaWaylandPopupGrab *grab);
 
-MetaWaylandSurface *meta_wayland_popup_grab_get_top_popup (MetaWaylandPopupGrab *grab);
+gboolean meta_wayland_popup_grab_has_popups (MetaWaylandPopupGrab *grab);
 
-gboolean meta_wayland_pointer_grab_is_popup_grab (MetaWaylandPointerGrab *grab);
+MetaWaylandSurface *meta_wayland_popup_grab_get_top_popup (MetaWaylandPopupGrab *grab);
 
 MetaWaylandPopup *meta_wayland_popup_create (MetaWaylandPopupSurface *surface,
                                              MetaWaylandPopupGrab    *grab);
@@ -58,5 +58,3 @@ void meta_wayland_popup_destroy (MetaWaylandPopup *popup);
 void meta_wayland_popup_dismiss (MetaWaylandPopup *popup);
 
 MetaWaylandSurface *meta_wayland_popup_get_top_popup (MetaWaylandPopup *popup);
-
-#endif /* META_WAYLAND_POPUP_H */
