@@ -194,39 +194,45 @@ meta_wayland_seat_devices_updated (ClutterSeat        *clutter_seat,
 
 static MetaWaylandSurface *
 default_get_focus_surface (MetaWaylandEventHandler *handler,
-                           ClutterInputDevice      *device,
-                           ClutterEventSequence    *sequence,
+                           ClutterFocus            *focus,
                            gpointer                 user_data)
 {
   MetaWaylandSeat *seat = user_data;
+  MetaWaylandSurface *surface = NULL;
+  ClutterInputCapabilities caps;
+  ClutterInputDevice *device;
 
-  return meta_wayland_seat_get_current_surface (seat,
-                                                device,
-                                                sequence);
+  if (CLUTTER_IS_SPRITE (focus))
+    {
+      device = clutter_sprite_get_device (CLUTTER_SPRITE (focus));
+      caps = clutter_input_device_get_capabilities (device);
+
+      if (caps &
+          (CLUTTER_INPUT_CAPABILITY_POINTER |
+           CLUTTER_INPUT_CAPABILITY_TOUCHPAD |
+           CLUTTER_INPUT_CAPABILITY_TRACKBALL |
+           CLUTTER_INPUT_CAPABILITY_TRACKPOINT))
+        surface = meta_wayland_pointer_get_implicit_grab_surface (seat->pointer);
+    }
+
+  if (!surface)
+    surface = meta_wayland_seat_get_current_surface (seat, focus);
+
+  return surface;
 }
 
 static void
 default_focus (MetaWaylandEventHandler *handler,
-               ClutterInputDevice      *device,
-               ClutterEventSequence    *sequence,
+               ClutterFocus            *focus,
                MetaWaylandSurface      *surface,
                gpointer                 user_data)
 {
   MetaWaylandSeat *seat = user_data;
+  ClutterInputDevice *device;
+  ClutterEventSequence *sequence;
   ClutterInputCapabilities caps;
 
-  if (sequence)
-    {
-      if (surface != meta_wayland_touch_get_surface (seat->touch, sequence))
-        meta_wayland_touch_cancel (seat->touch);
-      return;
-    }
-
-  caps = clutter_input_device_get_capabilities (device);
-
-  if (caps &
-      (CLUTTER_INPUT_CAPABILITY_KEYBOARD |
-       CLUTTER_INPUT_CAPABILITY_TABLET_PAD))
+  if (CLUTTER_IS_KEY_FOCUS (focus))
     {
       if (meta_wayland_seat_has_keyboard (seat))
         meta_wayland_keyboard_set_focus (seat->keyboard, surface);
@@ -236,8 +242,22 @@ default_focus (MetaWaylandEventHandler *handler,
                                                   surface);
       meta_wayland_tablet_seat_set_pad_focus (seat->tablet_seat, surface);
       meta_wayland_text_input_set_focus (seat->text_input, surface);
+      return;
     }
 
+  g_assert (CLUTTER_IS_SPRITE (focus));
+  device = clutter_sprite_get_device (CLUTTER_SPRITE (focus));
+  sequence = clutter_sprite_get_sequence (CLUTTER_SPRITE (focus));
+
+  if (sequence)
+    {
+      if (surface != meta_wayland_touch_get_surface (seat->touch, sequence))
+        meta_wayland_touch_cancel (seat->touch);
+      return;
+    }
+
+  caps = clutter_input_device_get_capabilities (device);
+
   if (caps & CLUTTER_INPUT_CAPABILITY_TABLET_TOOL)
     {
       meta_wayland_tablet_seat_focus_surface (seat->tablet_seat,
@@ -301,6 +321,7 @@ meta_wayland_seat_new (MetaWaylandCompositor *compositor,
                               NULL);
 
   seat->text_input = meta_wayland_text_input_new (seat);
+  seat->pointer_warp = meta_wayland_pointer_warp_new (seat);
 
   meta_wayland_data_device_init (&seat->data_device, seat);
   meta_wayland_data_device_primary_init (&seat->primary_data_device, seat);
@@ -346,6 +367,7 @@ meta_wayland_seat_free (MetaWaylandSeat *seat)
   g_object_unref (seat->touch);
 
   meta_wayland_text_input_destroy (seat->text_input);
+  meta_wayland_pointer_warp_destroy (seat->pointer_warp);
 
   g_free (seat);
 }
@@ -552,6 +574,14 @@ void
 meta_wayland_seat_set_input_focus (MetaWaylandSeat    *seat,
                                    MetaWaylandSurface *surface)
 {
+  MetaContext *context =
+    meta_wayland_compositor_get_context (seat->compositor);
+  MetaBackend *backend = meta_context_get_backend (context);
+  ClutterStage *stage = CLUTTER_STAGE (meta_backend_get_stage (backend));
+  ClutterBackend *clutter_backend =
+    meta_backend_get_clutter_backend (backend);
+  ClutterKeyFocus *key_focus;
+
   if (seat->input_focus == surface)
     return;
 
@@ -572,9 +602,10 @@ meta_wayland_seat_set_input_focus (MetaWaylandSeat    *seat,
                           seat);
     }
 
+  key_focus = clutter_backend_get_key_focus (clutter_backend, stage);
+
   meta_wayland_input_invalidate_focus (seat->input_handler,
-                                       clutter_seat_get_keyboard (seat->clutter_seat),
-                                       NULL);
+                                       CLUTTER_FOCUS (key_focus));
 }
 
 MetaWaylandSurface *
@@ -588,8 +619,7 @@ meta_wayland_seat_get_grab_info (MetaWaylandSeat       *seat,
                                  MetaWaylandSurface    *surface,
                                  uint32_t               serial,
                                  gboolean               require_pressed,
-                                 ClutterInputDevice   **device_out,
-                                 ClutterEventSequence **sequence_out,
+                                 ClutterSprite        **sprite_out,
                                  float                 *x,
                                  float                 *y)
 {
@@ -599,14 +629,10 @@ meta_wayland_seat_get_grab_info (MetaWaylandSeat       *seat,
 
       sequence = meta_wayland_touch_find_grab_sequence (seat->touch,
                                                         surface,
-                                                        serial);
+                                                        serial,
+                                                        sprite_out);
       if (sequence)
         {
-          if (device_out)
-            *device_out = clutter_seat_get_pointer (seat->clutter_seat);
-          if (sequence_out)
-            *sequence_out = sequence;
-
           meta_wayland_touch_get_press_coords (seat->touch, sequence, x, y);
           return TRUE;
         }
@@ -617,25 +643,17 @@ meta_wayland_seat_get_grab_info (MetaWaylandSeat       *seat,
                                           surface,
                                           serial,
                                           require_pressed,
-                                          device_out,
+                                          sprite_out,
                                           x, y))
-    {
-      if (sequence_out)
-        *sequence_out = NULL;
-      return TRUE;
-    }
+    return TRUE;
 
   if (meta_wayland_tablet_seat_get_grab_info (seat->tablet_seat,
                                               surface,
                                               serial,
                                               require_pressed,
-                                              device_out,
+                                              sprite_out,
                                               x, y))
-    {
-      if (sequence_out)
-        *sequence_out = NULL;
-      return TRUE;
-    }
+    return TRUE;
 
   return FALSE;
 }
@@ -688,10 +706,19 @@ meta_wayland_seat_get_input (MetaWaylandSeat *seat)
 }
 
 MetaWaylandSurface *
-meta_wayland_seat_get_current_surface (MetaWaylandSeat      *seat,
-                                       ClutterInputDevice   *device,
-                                       ClutterEventSequence *sequence)
+meta_wayland_seat_get_current_surface (MetaWaylandSeat *seat,
+                                       ClutterFocus    *focus)
 {
+  ClutterInputDevice *device;
+  ClutterEventSequence *sequence;
+
+  if (CLUTTER_IS_KEY_FOCUS (focus))
+    return seat->input_focus;
+
+  g_assert (CLUTTER_IS_SPRITE (focus));
+  device = clutter_sprite_get_device (CLUTTER_SPRITE (focus));
+  sequence = clutter_sprite_get_sequence (CLUTTER_SPRITE (focus));
+
   if (sequence)
     {
       return meta_wayland_touch_get_surface (seat->touch, sequence);
@@ -702,11 +729,6 @@ meta_wayland_seat_get_current_surface (MetaWaylandSeat      *seat,
 
       caps = clutter_input_device_get_capabilities (device);
 
-      if (caps &
-          (CLUTTER_INPUT_CAPABILITY_KEYBOARD |
-           CLUTTER_INPUT_CAPABILITY_TABLET_PAD))
-        return seat->input_focus;
-
       if (caps & CLUTTER_INPUT_CAPABILITY_TABLET_TOOL)
         {
           return meta_wayland_tablet_seat_get_current_surface (seat->tablet_seat,
@@ -719,17 +741,7 @@ meta_wayland_seat_get_current_surface (MetaWaylandSeat      *seat,
            CLUTTER_INPUT_CAPABILITY_TRACKBALL |
            CLUTTER_INPUT_CAPABILITY_TRACKPOINT))
         {
-          MetaWaylandSurface *implicit_grab_surface;
-
-          implicit_grab_surface =
-            meta_wayland_pointer_get_implicit_grab_surface (seat->pointer);
-
-          if (implicit_grab_surface &&
-              meta_wayland_input_is_current_handler (seat->input_handler,
-                                                     seat->default_handler))
-            return implicit_grab_surface;
-          else
-            return meta_wayland_pointer_get_current_surface (seat->pointer);
+          return meta_wayland_pointer_get_current_surface (seat->pointer);
         }
     }
 
